@@ -10,6 +10,7 @@ compas
     :titlesonly:
 
     compas.artists
+    compas.colors
     compas.data
     compas.datastructures
     compas.files
@@ -40,22 +41,22 @@ __copyright__ = 'Copyright 2014-2019 - Block Research Group, ETH Zurich'
 __license__ = 'MIT License'
 __email__ = 'vanmelet@ethz.ch'
 
-__version__ = '1.13.3'
+__version__ = '1.15.0'
 
 version = LooseVersion(compas.__version__)
 versionstring = version.vstring.split('-')[0]
 
-HERE = os.path.dirname(__file__)
+HERE = compas._os.realpath(os.path.dirname(__file__))
 """str: Path to the location of the compas package."""
 
 HOME = compas._os.absjoin(HERE, '../..')
-"""str: Path to the root of the local repo."""
+"""str: Path to the root of the local installation."""
 
-DATA = compas._os.absjoin(HERE, '../../data')
-"""str: Path to the data folder of the local repo."""
+DATA = compas._os.absjoin(HERE, 'data', 'samples')
+"""str: Path to the data folder of the local installation."""
 
 TEMP = compas._os.absjoin(HERE, '../../temp')
-"""str: Path to the temp folder of the local repo."""
+"""str: Path to the temp folder of the local installation."""
 
 APPDATA = compas._os.user_data_dir('COMPAS', 'compas-dev', roaming=True)
 """str: Path to the COMPAS directory in APPDATA."""
@@ -75,6 +76,9 @@ for formatting numbers.
 Float formatting (``'<x>f'``) and integer formatting (``'d'``) specifiers are supported.
 """
 
+PY2 = compas._os.PY2
+"""bool: True if the current Python version is 2.x, False otherwise."""
+
 PY3 = compas._os.PY3
 """bool: True if the current Python version is 3.x, False otherwise."""
 
@@ -123,7 +127,7 @@ except Exception:
 
 
 __all__ = [
-    'WINDOWS', 'LINUX', 'OSX', 'MONO', 'IPY', 'RHINO', 'BLENDER',
+    'WINDOWS', 'LINUX', 'OSX', 'MONO', 'IPY', 'RHINO', 'BLENDER', 'PY2', 'PY3',
     'is_windows', 'is_linux', 'is_osx', 'is_mono', 'is_ironpython', 'is_rhino', 'is_blender',
     'set_precision',
     'get',
@@ -171,27 +175,19 @@ def get(filename):
     ----------
     filename : str
         The name of the data file.
-        The following are available.
-
-        * boxes.obj
-        * faces.obj
-        * fink.obj
-        * hypar.obj
-        * lines.obj
-        * saddle.obj
 
     Returns
     -------
     str
         The full path to the specified file.
+        The path can be local or remote depending on availability.
 
     Notes
     -----
     The file name should be specified relative to the **COMPAS** sample data folder.
-    This folder is only locally available if you installed **COMPAS** from source,
-    or if you are working directly with the source.
-    In all other cases, the function will get the corresponding files directly from
-    the GitHub repo, at https://raw.githubusercontent.com/compas-dev/compas/main/data
+    If the requested file is not locally available in the sample data folder,
+    the function will get the corresponding file path from
+    the GitHub repo, at https://raw.githubusercontent.com/compas-dev/compas/main/samples
 
     Examples
     --------
@@ -216,7 +212,7 @@ def get(filename):
     if os.path.exists(localpath):
         return localpath
     else:
-        return "https://github.com/compas-dev/compas/raw/main/data/{}".format(filename)
+        return "https://raw.githubusercontent.com/compas-dev/compas/main/src/compas/data/samples/{}".format(filename)
 
 
 def get_bunny(localstorage=None):
