@@ -4,8 +4,9 @@ from __future__ import division
 
 from abc import abstractmethod
 
-from compas.utilities import is_color_rgb
+from compas.colors import Color
 from .artist import Artist
+from .colordict import ColorDict
 
 
 class VolMeshArtist(Artist):
@@ -13,77 +14,92 @@ class VolMeshArtist(Artist):
 
     Parameters
     ----------
-    volmesh : :class:`compas.datastructures.VolMesh`
+    volmesh : :class:`~compas.datastructures.VolMesh`
         A COMPAS volmesh.
 
-    Class Attributes
-    ----------------
-    default_vertexcolor : tuple
-        The default color for vertices that do not have a specified color.
-    default_edgecolor : tuple
-        The default color for edges that do not have a specified color.
-    default_facecolor : tuple
-        The default color for faces that do not have a specified color.
-    default_cellcolor : tuple
-        The default color for cells that do not have a specified color.
-
     Attributes
     ----------
-    volmesh : :class:`compas.datastructures.VolMesh`
+    volmesh : :class:`~compas.datastructures.VolMesh`
         The COMPAS volmesh associated with the artist.
-    vertices : list
+    vertices : list[int]
         The list of vertices to draw.
         Default is a list of all vertices of the volmesh.
-    edges : list
+    edges : list[tuple[int, int]]
         The list of edges to draw.
         Default is a list of all edges of the volmesh.
-    faces : list
+    faces : list[int]
         The list of faces to draw.
         Default is a list of all faces of the volmesh.
-    cells : list
+    cells : list[int]
         The list of cells to draw.
         Default is a list of all cells of the volmesh.
-    vertex_xyz : dict[int, tuple(float, float, float)]
-        Mapping between vertices and their view coordinates.
-        The default view coordinates are the actual coordinates of the vertices of the volmesh.
-    vertex_color : dict[int, tuple(int, int, int)]
-        Mapping between vertices and RGB color values.
-        The colors have to be integer tuples with values in the range ``0-255``.
-        Missing vertices get the default vertex color (``~VolMeshArtist.default_vertexcolor``).
+    vertex_xyz : dict[int, list[float]]
+        The view coordinates of the vertices.
+        By default, the actual vertex coordinates are used.
+    vertex_color : dict[int, :class:`~compas.colors.Color`]
+        Mapping between vertices and colors.
+        Missing vertices get the default vertex color: :attr:`default_vertexcolor`.
+    edge_color : dict[tuple[int, int], :class:`~compas.colors.Color`]
+        Mapping between edges and colors.
+        Missing edges get the default edge color: :attr:`default_edgecolor`.
+    face_color : dict[int, :class:`~compas.colors.Color`]
+        Mapping between faces and colors.
+        Missing faces get the default face color: :attr:`default_facecolor`.
+    cell_color : dict[int, :class:`~compas.colors.Color`]
+        Mapping between cells and colors.
+        Missing cells get the default cell color: :attr:`default_facecolor`.
     vertex_text : dict[int, str]
         Mapping between vertices and text labels.
-        Missing vertices are labelled with the corresponding vertex identifiers.
-    edge_color : dict[tuple(int, int), tuple(int, int, int)]
-        Mapping between edges and RGB color values.
-        The colors have to be integer tuples with values in the range ``0-255``.
-        Missing edges get the default edge color (``~VolMeshArtist.default_edgecolor``).
-    edge_text : dict[tuple(int, int), str]
+    edge_text : dict[tuple[int, int], str]
         Mapping between edges and text labels.
-        Missing edges are labelled with the corresponding edge identifiers.
-    face_color : dict[tuple, tuple(int, int, int)]
-        Mapping between faces and RGB color values.
-        The colors have to be integer tuples with values in the range ``0-255``.
-        Missing faces get the default face color (``~VolMeshArtist.default_facecolor``).
-    face_text : dict[tuple, str]
-        Mapping between faces and text labels.
-        Missing faces are labelled with the corresponding face identifiers.
-    cell_color : dict[int, tuple(int, int, int)]
-        Mapping between cells and RGB color values.
-        The colors have to be integer tuples with values in the range ``0-255``.
-        Missing cells get the default cell color (``~VolMeshArtist.default_cellcolor``).
+    face_text : dict[int, str]
+        Mapping between faces and text lables.
     cell_text : dict[int, str]
-        Mapping between cells and text labels.
-        Missing cells are labelled with the corresponding cell identifiers.
+        Mapping between cells and text lables.
+
+    Class Attributes
+    ----------------
+    default_vertexcolor : :class:`~compas.colors.Color`
+        The default color of the vertices of the mesh that don't have a specified color.
+    default_edgecolor : :class:`~compas.colors.Color`
+        The default color of the edges of the mesh that don't have a specified color.
+    default_facecolor : :class:`~compas.colors.Color`
+        The default color of the faces of the mesh that don't have a specified color.
+    default_cellcolor : :class:`~compas.colors.Color`
+        The default color of the cells of the mesh that don't have a specified color.
 
     """
 
-    default_vertexcolor = (1, 1, 1)
-    default_edgecolor = (0, 0, 0)
-    default_facecolor = (0.8, 0.8, 0.8)
-    default_cellcolor = (1, 0, 0)
+    color = Color.from_hex('#0092D2').lightened(50)
 
-    def __init__(self, volmesh, **kwargs):
+    default_vertexcolor = Color.from_hex('#0092D2')
+    default_edgecolor = Color.from_hex('#0092D2')
+    default_facecolor = Color.from_hex('#0092D2').lightened(50)
+    default_cellcolor = Color.from_hex('#0092D2').lightened(50)
+
+    vertex_color = ColorDict()
+    edge_color = ColorDict()
+    face_color = ColorDict()
+    cell_color = ColorDict()
+
+    def __init__(self,
+                 volmesh,
+                 vertices=None,
+                 edges=None,
+                 faces=None,
+                 cells=None,
+                 vertexcolor=None,
+                 edgecolor=None,
+                 facecolor=None,
+                 cellcolor=None,
+                 **kwargs):
         super(VolMeshArtist, self).__init__()
+
+        self._default_vertexcolor = None
+        self._default_edgecolor = None
+        self._default_facecolor = None
+        self._default_cellcolor = None
+
         self._volmesh = None
         self._vertices = None
         self._edges = None
@@ -98,8 +114,29 @@ class VolMeshArtist(Artist):
         self._edge_text = None
         self._face_text = None
         self._cell_text = None
+
+        self._vertexcollection = None
+        self._edgecollection = None
+        self._facecollection = None
+        self._cellcollection = None
+        self._vertexnormalcollection = None
+        self._facenormalcollection = None
+        self._vertexlabelcollection = None
+        self._edgelabelcollection = None
+        self._facelabelcollection = None
+        self._celllabelcollection = None
+
         self.volmesh = volmesh
 
+        self.vertices = vertices
+        self.edges = edges
+        self.faces = faces
+        self.cells = cells
+        self.vertex_color = vertexcolor
+        self.edge_color = edgecolor
+        self.face_color = facecolor
+        self.cell_color = cellcolor
+
     @property
     def volmesh(self):
         return self._volmesh
@@ -159,58 +196,6 @@ class VolMeshArtist(Artist):
     def vertex_xyz(self, vertex_xyz):
         self._vertex_xyz = vertex_xyz
 
-    @property
-    def vertex_color(self):
-        if not self._vertex_color:
-            self._vertex_color = {vertex: self.artist.default_vertexcolor for vertex in self.volmesh.vertices()}
-        return self._vertex_color
-
-    @vertex_color.setter
-    def vertex_color(self, vertex_color):
-        if isinstance(vertex_color, dict):
-            self._vertex_color = vertex_color
-        elif is_color_rgb(vertex_color):
-            self._vertex_color = {vertex: vertex_color for vertex in self.volmesh.vertices()}
-
-    @property
-    def edge_color(self):
-        if not self._edge_color:
-            self._edge_color = {edge: self.artist.default_edgecolor for edge in self.volmesh.edges()}
-        return self._edge_color
-
-    @edge_color.setter
-    def edge_color(self, edge_color):
-        if isinstance(edge_color, dict):
-            self._edge_color = edge_color
-        elif is_color_rgb(edge_color):
-            self._edge_color = {edge: edge_color for edge in self.volmesh.edges()}
-
-    @property
-    def face_color(self):
-        if not self._face_color:
-            self._face_color = {face: self.artist.default_facecolor for face in self.volmesh.faces()}
-        return self._face_color
-
-    @face_color.setter
-    def face_color(self, face_color):
-        if isinstance(face_color, dict):
-            self._face_color = face_color
-        elif is_color_rgb(face_color):
-            self._face_color = {face: face_color for face in self.volmesh.faces()}
-
-    @property
-    def cell_color(self):
-        if not self._cell_color:
-            self._cell_color = {cell: self.artist.default_cellcolor for cell in self.volmesh.cells()}
-        return self._cell_color
-
-    @cell_color.setter
-    def cell_color(self, cell_color):
-        if isinstance(cell_color, dict):
-            self._cell_color = cell_color
-        elif is_color_rgb(cell_color):
-            self._cell_color = {cell: cell_color for cell in self.volmesh.cells()}
-
     @property
     def vertex_text(self):
         if not self._vertex_text:
@@ -277,16 +262,22 @@ class VolMeshArtist(Artist):
 
         Parameters
         ----------
-        vertices : list, optional
+        vertices : list[int], optional
             The vertices to include in the drawing.
             Default is all vertices.
-        color : tuple or dict, optional
+        color : tuple[float, float, float] | :class:`~compas.colors.Color` | dict[int, tuple[float, float, float] | :class:`~compas.colors.Color`], optional
             The color of the vertices,
             as either a single color to be applied to all vertices,
             or a color dict, mapping specific vertices to specific colors.
-        text : dict, optional
-            The text labels for the vertices
-            as a text dict, mapping specific vertices to specific text labels.
+        text : dict[int, str], optional
+            The text labels for the vertices as a text dict,
+            mapping specific vertices to specific text labels.
+
+        Returns
+        -------
+        list
+            The identifiers of the objects representing the vertices in the visualization context.
+
         """
         raise NotImplementedError
 
@@ -296,16 +287,22 @@ class VolMeshArtist(Artist):
 
         Parameters
         ----------
-        edges : list, optional
+        edges : list[tuple[int, int]], optional
             The edges to include in the drawing.
             Default is all edges.
-        color : tuple or dict, optional
+        color : tuple[float, float, float] | :class:`~compas.colors.Color` | dict[tuple[int, int], tuple[float, float, float] | :class:`~compas.colors.Color`], optional
             The color of the edges,
             as either a single color to be applied to all edges,
             or a color dict, mapping specific edges to specific colors.
-        text : dict, optional
-            The text labels for the edges
-            as a text dict, mapping specific edges to specific text labels.
+        text : dict[tuple[int, int], str], optional
+            The text labels for the edges as a text dict,
+            mapping specific edges to specific text labels.
+
+        Returns
+        -------
+        list
+            The identifiers of the objects representing the edges in the visualization context.
+
         """
         raise NotImplementedError
 
@@ -315,16 +312,22 @@ class VolMeshArtist(Artist):
 
         Parameters
         ----------
-        faces : list, optional
+        faces : list[int], optional
             The faces to include in the drawing.
             Default is all faces.
-        color : tuple or dict, optional
+        color : tuple[float, float, float] | :class:`~compas.colors.Color` | dict[int, tuple[float, float, float] | :class:`~compas.colors.Color`], optional
             The color of the faces,
             as either a single color to be applied to all faces,
             or a color dict, mapping specific faces to specific colors.
-        text : dict, optional
-            The text labels for the faces
-            as a text dict, mapping specific faces to specific text labels.
+        text : dict[int, str], optional
+            The text labels for the faces as a text dict,
+            mapping specific faces to specific text labels.
+
+        Returns
+        -------
+        list
+            The identifiers of the objects representing the faces in the visualization context.
+
         """
         raise NotImplementedError
 
@@ -334,31 +337,65 @@ class VolMeshArtist(Artist):
 
         Parameters
         ----------
-        cells : list, optional
+        cells : list[int], optional
             The cells to include in the drawing.
             Default is all cells.
-        color : tuple or dict, optional
+        color : tuple[float, float, float] | :class:`~compas.colors.Color` | dict[int, tuple[float, float, float] | :class:`~compas.colors.Color`], optional
             The color of the cells,
             as either a single color to be applied to all cells,
             or a color dict, mapping specific cells to specific colors.
-        text : dict, optional
-            The text labels for the cells
-            as a text dict, mapping specific cells to specific text labels.
+        text : dict[int, str], optional
+            The text labels for the cells as a text dict,
+            mapping specific cells to specific text labels.
+
+        Returns
+        -------
+        list
+            The identifiers of the objects representing the cells in the visualization context.
+
         """
         raise NotImplementedError
 
     @abstractmethod
     def clear_vertices(self):
+        """Clear the vertices of the mesh.
+
+        Returns
+        -------
+        None
+
+        """
         raise NotImplementedError
 
     @abstractmethod
     def clear_edges(self):
+        """Clear the edges of the mesh.
+
+        Returns
+        -------
+        None
+
+        """
         raise NotImplementedError
 
     @abstractmethod
     def clear_faces(self):
+        """Clear the faces of the mesh.
+
+        Returns
+        -------
+        None
+
+        """
         raise NotImplementedError
 
     @abstractmethod
     def clear_cells(self):
+        """Clear the cells of the mesh.
+
+        Returns
+        -------
+        None
+
+        """
         raise NotImplementedError
